/**
 * Copyright (c) 2008 - 2016 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */
import com.smaxe.uv.client.rtsp.IPresentation;
import com.smaxe.uv.client.rtsp.IVideo;
import com.smaxe.uv.client.rtsp.RtpPlaySession;
import com.smaxe.uv.client.rtsp.RtspClient;
import com.smaxe.uv.client.rtsp.video.AbstractVideo;
import com.smaxe.uv.client.rtsp.video.VideoFactory;
import com.smaxe.uv.media.core.MediaContainer;
import com.smaxe.uv.media.core.MediaTrackInfo;
import com.smaxe.uv.stream.MediaData;

import java.util.Arrays;

/**
 * <code>ExRtpPlayStream</code> - {@link RtspClient} usage example.
 * <p> Note:
 * <br> - The example shows how to play unicast/multicast RTP stream.
 * <br> - The played stream is saved to the local file. 
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtsp-client" target="_blank">JUV RTSP/RTP Client</a> 
 */
public final class ExRtpPlayStream extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     * @throws Exception if an exception occurred
     */
    public static void main(final String[] args) throws Exception
    {
        // NOTE:
        // you can get Evaluation Key at:
        // http://www.smaxe.com/order.jsf#request_evaluation_key
        // or buy at:
        // http://www.smaxe.com/order.jsf
        
        // Android-specific:
        // - please add permission to the AndroidManifest.xml : 
        // <uses-permission android:name="android.permission.INTERNET" />
        // - please use separate thread to connect to the server (not UI thread) : 
        // RtspClient#connect() connects to the remote server in the invocation thread,
        // so it causes NetworkOnMainThreadException on 4.0 (http://developer.android.com/reference/android/os/NetworkOnMainThreadException.html)
        com.smaxe.uv.client.rtsp.License.setKey("SET-YOUR-KEY");
        
        // presentation
        final String multicastSdp =
            "v=0\n" +
            "o=- 1414976727 3624060226 IN IP4 192.168.0.1\n" +
            "s=LIVECA~1\n" +
            "i=MP3 audio\n" +
            "b=AS:32\n" +
            "t=3624060226 3624061846\n" +
            "a=type:broadcast\n" +
            "a=tool:liveCaster v1.0b1\n" +
            "m=audio 50848 RTP/AVP 14\n" +
            "c=IN IP4 234.38.3.221/255";
        
        final String unicastSdp = 
            "v=0\n" +
            "o=- 0 0 IN IP4 127.0.0.1\n" +
            "s=No Name\n" +
            "c=IN IP4 127.0.0.1\n" +
            "t=0 0\n" +
            "a=tool:libavformat 57.25.100\n" +
            "m=video 6008 RTP/AVP 96\n" +
            "b=AS:256\n" +
            "a=rtpmap:96 H264/90000\n" +
            "a=fmtp:96 packetization-mode=1";
        
        // 
        final boolean multicast = false;
        // set true to record incoming stream, false - to log incoming data to console
        final boolean record = false;
        
        final RtspClient client = new RtspClient();
        
        // configure client
        client.configuration().put(RtspClient.Configuration.DATAGRAM_JITTER_BUFFER_MARGIN, 16 /*packets*/);
        client.configuration().put(RtspClient.Configuration.DATAGRAM_SO_ENABLE_CONFIGURATION, true);
        client.configuration().put(RtspClient.Configuration.DATAGRAM_SO_RCVBUF, 64 * 1024);
        
        // connect
        
        try
        {
            // get and print media stream presentation (SDP)
            final IPresentation presentation = RtspClient.createSdpPresentation(multicast ? multicastSdp : unicastSdp);
            
            System.out.println(presentation);
            
            // create 'PLAY' session
            final RtpPlaySession session = client.createRtpPlaySession(presentation);
            
            // set listener (optional)
            session.setListener(new RtpPlaySession.ListenerAdapter()
            {
                @Override
                public void onMediaTimeout(final MediaTrackInfo track, final int timeout)
                {
                    System.out.println("PlaySession.IListener#onMediaTimeout: " + track + " " + timeout);
                }
                
                @Override
                public void onPacketLoss(final MediaTrackInfo track, final long lost, final long total, final int margin)
                {
                    // NOTE: please use the method below to adjust jitter buffer margin
                    // session.setJitterBufferMargin(track, 12);
                    
                    System.out.println("PlaySession.IListener#onPacketLoss: " + track + " " + lost + " " + total);
                }
            });
            
            final MediaContainer[] containers = VideoFactory.findMediaContainer(presentation);
            if (record && containers.length == 0) throw new Exception("Presentation can't be written to the file, no appropriate container found");
            
            final MediaContainer container = record ? containers[0] : null;
            if (container != null) System.out.println("Selected media container is " + container);
            
            // NOTE: file extension is added by the "VideoFactory#create(*)" implementation
            final IVideo video = record ? VideoFactory.create("d:/rtsp", container) :
                new AbstractVideo()
                {
                    @Override
                    public void onPlay(final MediaTrackInfo[] tracks)
                    {
                        System.out.println("IVideo#onPlay:" + Arrays.toString(tracks));
                    }
                    
                    @Override
                    public void onMediaData(final MediaTrackInfo track, final MediaData data)
                    {
                        // NOTE: please use the method below to get media data content
                        // ByteArray payload = MediaDataFactory.getMediaDataPayload(data);                    
                        
                        System.out.println("IVideo#onMediaData: "  + data + " " + track);
                    }
                    
                    @Override
                    public void onTeardown()
                    {
                        System.out.println("IVideo#onTeardown");
                    }
                };
            
            session.setup();
            
            session.play(video);
            
            Thread.sleep(10 * 1000);
            
            session.teardown();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            client.release();
        }
    }
    
    /**
     * Constructor.
     */
    private ExRtpPlayStream()
    {
    }
}